@@ -479,7 +479,7 @@ static int intersectCurves(NSPoint pc1[4], const NSPoint pc2[4], NSPoint *points
 
     [curve setWidth:width];
     [curve setColor:color];
-    [curve setLocked:isLocked];
+    [curve setLocked:NO];
     [curve setVertices:p0 :p1 :p2 :p3];
     [curve setSelected:[self isSelected]];
     return curve;
