@@ -5,7 +5,7 @@
  * Author:   Georg Fleischmann
  *
  * created:  1996-01-29
- * modified: 2005-04-25 (clean up)
+ * modified: 2005-09-23
  *
  * the group is implemented as follows:
  * - all objects are stored in a list with absolute coordinates
@@ -136,11 +136,87 @@
     for (i=[list count]-1; i>=0; i--)
     {	id	g = [list objectAtIndex:i];
 
-        if ( [g isKindOfClass:[VGroup class]] || [g isKindOfClass:[VPath class]] )
+        if ( [g isKindOfClass:[VGroup class]] ) // || [g isKindOfClass:[VPath class]] 
             cnt += [g countRecursive];
     }
     return cnt;
 }
+/* created 2005-08-19
+ */
+- (id)recursiveObjectAtIndex:(int)ix
+{   int	i, cnt = [list count], cCnt = 0;
+
+    for (i=0; i < cnt; i++)
+    {	id	g = [list objectAtIndex:i];
+
+        if ([g isKindOfClass:[VGroup class]])
+        { int	gCnt = [g countRecursive];
+
+            if (ix >= cCnt && ix < cCnt+gCnt) // obj is inside g
+                return [g recursiveObjectAtIndex:(ix-cCnt)];
+            cCnt += [g countRecursive];
+        }
+        else if (cCnt == ix)
+            return g;
+        else
+            cCnt++;
+    }
+    return nil;
+}
+- (void)recursiveRemoveObjectAtIndex:(int)ix
+{   int	i, cnt = [list count], cCnt = 0;
+
+    for (i=0; i < cnt; i++)
+    {	id	g = [list objectAtIndex:i];
+
+        if ([g isKindOfClass:[VGroup class]])
+        {   int	gCnt = [g countRecursive];
+
+            if (ix >= cCnt && ix < cCnt+gCnt) // obj is inside g
+            {   [g recursiveRemoveObjectAtIndex:(ix-cCnt)];
+                return;
+            }
+            cCnt += [g countRecursive];
+        }
+        else if (cCnt == ix)
+        {   [list removeObjectAtIndex:i];
+            return;
+        }
+        else
+            cCnt++;
+    }
+}
+- (void)recursiveInsertObject:(id)obj atIndex:(int)ix
+{   int	i, cnt = [list count], cCnt = 0;
+
+    if (ix >= [self countRecursive])
+    {
+        if ([[list objectAtIndex:cnt-1] isKindOfClass:[VGroup class]]) // last obj is a group
+            [[[list objectAtIndex:cnt-1] list] addObject:obj];
+        else
+            [list addObject:obj];
+    }
+
+    for (i=0; i < cnt; i++)
+    {	id	g = [list objectAtIndex:i];
+
+        if ([g isKindOfClass:[VGroup class]])
+        {   int	gCnt = [g countRecursive];
+
+            if (ix >= cCnt && ix < cCnt+gCnt) // obj is inside g
+            {   [g recursiveInsertObject:obj atIndex:(ix-cCnt)];
+                return;
+            }
+            cCnt += [g countRecursive];
+        }
+        else if (cCnt == ix)
+        {   [list insertObject:obj atIndex:i];
+            return;
+        }
+        else
+            cCnt++;
+    }
+}
 
 /* created:  1995-09-24
  * modified: 
@@ -1123,6 +1199,7 @@
     return group;
 }
 
+#if 0
 - getListOfObjectsSplittedFrom:(NSPoint*)pArray :(int)iCnt
 {   NSMutableArray	*splitList = [NSMutableArray array], *spList = nil;
     int			i, j, cnt = [list count];
@@ -1147,10 +1224,10 @@
 }
 
 - (int)getIntersections:(NSPoint**)ppArray with:g
-{   int		i, j, iCnt = 0;
+{   int		i, j, iCnt = 0, ptsCnt = Min(100, [self numPoints]);
     NSPoint	*pts;
 
-    *ppArray = malloc(([list count]*9) * sizeof(NSPoint));
+    *ppArray = malloc(ptsCnt * sizeof(NSPoint));
     for (i=[list count]-1; i>=0; i--)
     {	id	gp = [list objectAtIndex:i];
         int	cnt, oldCnt = iCnt;
@@ -1159,6 +1236,9 @@
             continue;
 
         cnt = [gp getIntersections:&pts with:g];	/* line, arc, curve */
+        if (iCnt+cnt >= ptsCnt)
+            *ppArray = realloc(*ppArray, (ptsCnt+=cnt*2) * sizeof(NSPoint));
+
         for (j=0; j<cnt; j++)
         {
             if ( !pointInArray(pts[j], *ppArray, oldCnt) )
@@ -1217,6 +1297,7 @@
 
     return iCnt;
 }
+#endif
 
 - (float)sqrDistanceGraphic:g
 {   int		i;
@@ -1299,15 +1380,15 @@
 
     return newGroup;
 }*/
-
+#if 0
 - getIntersectionsAndSplittedObjects:(NSPoint**)ppArray :(int*)iCnt with:g
-{   int			i, j, lCnt = [list count];
+{   int			i, j, lCnt = [list count], ptsCnt = Min(100, [self numPoints]);
     NSPoint		*pts;
     NSRect		gBounds = [g bounds];
     NSMutableArray	*splitList = [NSMutableArray array], *spList = nil;
 
     *iCnt = 0;
-    *ppArray = malloc(([list count]*9) * sizeof(NSPoint));
+    *ppArray = malloc(ptsCnt * sizeof(NSPoint));
 //    for (i=[list count]-1; i>=0; i--)
     for (i=0; i<lCnt; i++)
     {	id	gp = [list objectAtIndex:i];
@@ -1335,6 +1416,10 @@
             else
                 [splitList addObject:[[gp copy] autorelease]];
         }
+
+        if ((*iCnt)+cnt >= ptsCnt)
+            *ppArray = realloc(*ppArray, (ptsCnt+=cnt*2) * sizeof(NSPoint));
+
         for (j=0; j<cnt; j++)
         {
             if ( !pointInArray(pts[j], *ppArray, oldCnt) )
@@ -1482,6 +1567,7 @@
     [pool release];
     return ng;
 }
+#endif
 
 - (void)writeFilesToDirectory:(NSString*)directory
 {   int	i;
