@@ -1,10 +1,10 @@
 /* VImage.m
  *
- * Copyright (C) 1996-2004 by vhf interservice GmbH
+ * Copyright (C) 1996-2005 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1998-03-22
- * modified: 2004-10-06
+ * modified: 2005-04-25 (clean up)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -484,7 +484,7 @@ static VImageFileType fileTypeForTIFFCompression(NSTIFFCompression tiffCompressi
 {
     if (!flag)
         selectedKnob = -1;
-    return [super setSelected:flag];
+    [super setSelected:flag];
 }
 
 - (void)scale:(float)x :(float)y withCenter:(NSPoint)cp
@@ -1096,7 +1096,7 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
                 [((thumbImage && VHFIsDrawingToScreen())?thumbImage:image) setSize:bounds.size];
             else
             {
-                p.x = origin.x + floor((bounds.size.width - currentSize.width) / 2.0 + 0.5);
+                p.x = origin.x + floor((bounds.size.width  - currentSize.width)  / 2.0 + 0.5);
                 p.y = origin.y + floor((bounds.size.height - currentSize.height) / 2.0 + 0.5);
             }
         }
@@ -1649,12 +1649,13 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
         unsigned char		*data[5], *cmykplanes[5];
         NSImage			*cmykImage;
         NSBitmapImageRep	*cmykbitmap;
-        NSString		*cmykFilename = nil;
+        NSString		*cmykFilename = nil, *string;
 
         if (![VImage isAcceptedFileType:fileType])
             fileType = VImageTIFF_None;	// default
 
-        cmykFilename = [NSString stringWithFormat:@"%@/%@_cmyk.tiff", [directory stringByDeletingLastPathComponent], [imageFile stringByDeletingPathExtension]];
+        string = [NSString stringWithFormat:@"%@_cmyk.tiff", [imageFile stringByDeletingPathExtension]];
+        cmykFilename = vhfPathWithPathComponents([directory stringByDeletingLastPathComponent], string, nil);
 
         if (![fileManager isWritableFileAtPath:[directory stringByDeletingLastPathComponent]])
         {
