@@ -80,7 +80,7 @@
 
     [line setWidth:width];
     [line setSelected:isSelected];
-    [line setLocked:isLocked];
+    [line setLocked:NO];
     [line setColor:color];
     [line setVertices:p0 :p1];
     return line;
