@@ -1,11 +1,11 @@
 /* dvPasteboard.m
  * Pasteboard additions for Cenon DocView class
  *
- * Copyright (C) 1997-2003 by vhf interservice GmbH + vhf Computer GmbH
+ * Copyright (C) 1997-2005 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1997
- * modified: 2003-06-26
+ * modified: 2005-09-23
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -32,6 +32,10 @@
 #include "LayerObject.h"
 #include "graphicsUndo.subproj/undo.h"
 
+#ifdef __APPLE__
+#   define PB_CANSERVE_PDF YES
+#endif
+
 @implementation DocView(NSPasteboard)
 
 /* Methods to search through Pasteboard types lists. */
@@ -81,24 +85,26 @@ NSString *e2ForeignPasteType(NSArray *types)
  * for pasting.  Cenon prefers its own type, then it prefers Text,
  * then something NSImage can handle.
  */
-NSString *e2DrawPasteType(NSArray *types)
+NSString *e2CenonPasteType(NSArray *types)
 {
     if (e2IncludesType(types, e2PboardType))
         return e2PboardType;
     return e2ForeignPasteType(types);
 }
 
+NSArray *e2TypesCenonExports(void)
+{   static	NSArray	*exportList = nil;
 
-#if 0
-NSArray *e2TypesDrawExports(void)
-{   static NSArray * exportList = nil;
     if (!exportList)
     {
+#ifdef PB_CANSERVE_PDF
+        exportList = [[NSArray allocWithZone:NSDefaultMallocZone()] initWithObjects:e2PboardType, NSPDFPboardType, NSPostScriptPboardType, NSTIFFPboardType, nil];
+#else
         exportList = [[NSArray allocWithZone:NSDefaultMallocZone()] initWithObjects:e2PboardType, NSPostScriptPboardType, NSTIFFPboardType, nil];
+#endif
     }
     return exportList;
 }
-#endif
 
 /* Lazy Pasteboard evaluation handler */
 
@@ -122,11 +128,10 @@ NSArray *e2TypesDrawExports(void)
  * to perform this very function).
  */
 + (void)convert:(NSUnarchiver *)unarchiver to:(NSString *)type using:(SEL)writer toPasteboard:(NSPasteboard *)pb
-{
-    NSWindow		*w;
+{   NSWindow		*w;
     NSMutableArray	*array;
     NSData		*data;
-    DocView	*scrapper;
+    DocView		*scrapper;
     NSRect		scrapperFrame = {{0.0, 0.0}, {11.0*72.0, 14.0*72.0}};
     static NSZone	*scrapperZone = NULL;
 
@@ -136,6 +141,7 @@ NSArray *e2TypesDrawExports(void)
     if (unarchiver)
     {
         scrapper = [[DocView allocWithZone:scrapperZone] initWithFrame:scrapperFrame];
+        [scrapper initView];	// init scale etc.
         [unarchiver setObjectZone:scrapperZone];
         array = [[NSMutableArray allocWithZone:scrapperZone] initWithArray:[unarchiver decodeObject]];
         scrapperFrame = [scrapper boundsOfArray:array];
@@ -143,11 +149,14 @@ NSArray *e2TypesDrawExports(void)
         scrapperFrame.size.height += scrapperFrame.origin.y;
         scrapperFrame.origin.x = scrapperFrame.origin.y = 0.0;
         [scrapper setFrameSize:(NSSize){ scrapperFrame.size.width, scrapperFrame.size.height }];
-        w = [[NSWindow allocWithZone:scrapperZone] initWithContentRect:scrapperFrame styleMask:NSBorderlessWindowMask backing:NSBackingStoreNonretained defer:NO];
+        w = [[NSWindow allocWithZone:scrapperZone] initWithContentRect:scrapperFrame
+                                                             styleMask:NSBorderlessWindowMask
+                                                               backing:NSBackingStoreNonretained
+                                                                 defer:NO];
         [w setContentView:scrapper];
-        data = [scrapper performSelector:writer withObject:array];
-        [pb setData:data forType:type];
-        //[array removeAllObjects];
+        if ((data = [scrapper performSelector:writer withObject:array]))
+            [pb setData:data forType:type];
+        [scrapper release];
         [array release];
         [w release];
     }
@@ -160,33 +169,40 @@ NSArray *e2TypesDrawExports(void)
  * the Pasteboard (which is in the internal format) is taken out and loaded
  * into an NSData, then convert:to:using:toPasteboard: is called. This
  * returns self if successful, nil otherwise.
+ * modified: 2005-09-23
  */
-+ (void)pasteboard:(NSPasteboard *)sender provideDataForType:(NSString *)type
++ (void)pasteboard:(NSPasteboard*)sender provideDataForType:(NSString*)type
 {
-    if ([type isEqual:NSPostScriptPboardType] || [type  isEqual:NSTIFFPboardType])
-    {
-	NSData *pbData;
-	
-	if ((pbData = [sender dataForType:e2PboardType]))
-        {
-	    NSUnarchiver *unarchiver = [[NSUnarchiver allocWithZone:(NSZone *)[(NSObject *)self zone]] initForReadingWithData:pbData];
-	    
-	    if (unarchiver)
+    if ( [e2TypesCenonExports() containsObject:type] )
+    {   NSData	*pbData;
+
+        if ((pbData = [sender dataForType:e2PboardType]))
+        {   NSUnarchiver *unarchiver = [[NSUnarchiver allocWithZone:(NSZone *)[(NSObject *)self zone]]
+                                       initForReadingWithData:pbData];
+
+            if (unarchiver)
             {
-		if ([type  isEqual:NSPostScriptPboardType])
+                if ([type isEqual:NSPostScriptPboardType])
                 {
-		    [self convert:unarchiver to:type using:@selector(dataForEPSUsingList:) toPasteboard:sender];
-		}
-                else
+                    [self convert:unarchiver to:type using:@selector(dataForEPSUsingList:)
+                     toPasteboard:sender];
+                }
+#ifdef PB_CANSERVE_PDF
+                else if ([type isEqual:NSPDFPboardType])
                 {
-		    // So it must be "NSTIFFPboardType"...
-		    [self convert:unarchiver to:type using:@selector(dataForTIFFUsingList:) toPasteboard:sender];
-		}
-		[unarchiver release];
-	    }
-//	    [sender deallocatePasteboardData:pbData];
-	}
-	
+                    [self convert:unarchiver to:type using:@selector(dataForPDFUsingList:)
+                     toPasteboard:sender];
+                }
+#endif
+                else	// So it must be "NSTIFFPboardType"...
+                {
+                    [self convert:unarchiver to:type using:@selector(dataForTIFFUsingList:)
+                     toPasteboard:sender];
+                }
+                [unarchiver release];
+            }
+            //[sender deallocatePasteboardData:pbData];
+        }
     }
 }
 
@@ -198,63 +214,74 @@ NSArray *e2TypesDrawExports(void)
  * be equal to the bounding box of the objects in the glist (NOT the
  * bounds of the view).
  */
-- (NSData *)dataForEPS
-{   NSRect bbox;
-    NSData *data = nil;
-
-//    if (([glist count] == 1) && [[glist objectAtIndex:0] canEmitEPS])
-//    {
-//        data = [[glist objectAtIndex:0] dataForEPS];
-//    } else
-    {
-        bbox = [self boundsOfArray:layerList];
-        data = [self dataWithEPSInsideRect:bbox];
-    }
+- (NSData*)dataForEPS
+{   NSRect	bbox;
+    NSData	*data = nil;
 
+    bbox = [self boundsOfArray:layerList];
+    data = [self dataWithEPSInsideRect:bbox];
     return data;
 }
+#ifdef PB_CANSERVE_PDF
+- (NSData*)dataForPDF
+{   NSRect	bbox;
+    NSData	*data = nil;
+
+    bbox = [self boundsOfArray:layerList];
+    data = [self dataWithPDFInsideRect:bbox];
+    return data;
+}
+#endif
 
 /*
  * This is the same as dataForEPS, but it lets you specify the list
  * of Graphics you want to generate PostScript for (does its job by swapping
  * the glist for the list you provide temporarily).
  */
-
 - (NSData *)dataForEPSUsingList:list
-{
-    NSData *data = nil;
-#if 0
-    NSMutableArray *savedglist;
-    
-    savedglist = glist;
-    glist = list;
+{   NSData	*data = nil;
+    int		l;
+
+    // FIXME: save current layer list
+    for (l=0; l<[list count]; l++)
+        [self addList:[list objectAtIndex:l] toLayerAtIndex:0];
     data = [self dataForEPS];
-    glist = savedglist;
-#endif
+    // FIXME: restore layer list
+
     return data;
 }
+#ifdef PB_CANSERVE_PDF
+- (NSData*)dataForPDFUsingList:list
+{   int	l;
+
+    for (l=0; l<[list count]; l++)
+        [self addList:[list objectAtIndex:l] toLayerAtIndex:0];
+    return [self dataForPDF];
+}
+#endif
 
 /*
  * Images all of the objects in the glist and writes out the result in
- * the Tagged VImage File Format (TIFF).  The image will not have alpha in it.
+ * the Tagged VImage File Format (TIFF). The image will not have alpha in it.
  */
+- (NSData*)dataForTIFF
+{   NSData	*data = nil;
 
-- (NSData *)dataForTIFF
-{
-    NSData *data = nil;
 #if 0
-    NSImage *tiffCache;
-    
-    if (([glist count] == 1) && [[glist objectAtIndex:0] canEmitTIFF]) {
-	data = [[glist objectAtIndex:0] dataForTIFF];
-    } else {
-        tiffCache = [[NSImage allocWithZone:[self zone]] initWithSize:[self getBBoxOfArray:glist].size];
-	[self cacheList:glist into:tiffCache withTransparentBackground:NO];
-	data = [tiffCache TIFFRepresentationUsingCompression:NSTIFFCompressionLZW factor:NSTIFFCompressionJPEG];
-	[tiffCache release];
-    }
+    NSImage	*tiffCache;
 
-#endif 
+    if (([glist count] == 1) && [[glist objectAtIndex:0] canEmitTIFF])
+    {
+        data = [[glist objectAtIndex:0] dataForTIFF];
+    }
+    else
+    {
+        tiffCache = [[NSImage allocWithZone:[self zone]] initWithSize:[self getBBoxOfArray:glist].size];
+        [self cacheList:glist into:tiffCache withTransparentBackground:NO];
+        data = [tiffCache TIFFRepresentationUsingCompression:NSTIFFCompressionLZW factor:NSTIFFCompressionJPEG];
+        [tiffCache release];
+    }
+#endif
     return data;
 }
 
@@ -263,14 +290,11 @@ NSArray *e2TypesDrawExports(void)
  * of Graphics you want to generate TIFF for (does its job by swapping
  * the glist for the list you provide temporarily).
  */
-
 - (NSData *)dataForTIFFUsingList:list
-{
-    NSData *data = nil;
+{   NSData		*data = nil;
 #if 0
-    NSMutableArray *savedglist;
-    
-    savedglist = glist;
+    NSMutableArray	*savedglist = glist;
+
     glist = list;
     data = [self dataForTIFF];
     glist = savedglist;
@@ -279,13 +303,18 @@ NSArray *e2TypesDrawExports(void)
 }
 
 
-/* Writing the selection to an NSData */
-
+/* Writing the selection to an NSData
+ */
 - (NSData *)copySelectionAsEPS
 {
     return ([slayList count]) ? [self dataForEPSUsingList:slayList] : nil;
 }
-
+#ifdef PB_CANSERVE_PDF
+- (NSData *)copySelectionAsPDF
+{
+    return ([slayList count]) ? [self dataForPDFUsingList:slayList] : nil;
+}
+#endif
 - (NSData *)copySelectionAsTIFF
 {
     return ([slayList count]) ? [self dataForTIFFUsingList:slayList] : nil;
@@ -302,10 +331,10 @@ NSArray *e2TypesDrawExports(void)
 
 /* Pasteboard-related target/action methods */
 
-- (void)cut:(id)sender
 /*
  * Calls copy: then delete:.
  */
+- (void)cut:(id)sender
 {   id change;
 
     if ([slayList count] > 0)
@@ -325,10 +354,10 @@ NSArray *e2TypesDrawExports(void)
 
     if ([slayList count])
     {
-	[self copyToPasteboard:[NSPasteboard generalPasteboard]];
-	lastPastedChangeCount = [[NSPasteboard generalPasteboard] changeCount];
-	lastCopiedChangeCount = [[NSPasteboard generalPasteboard] changeCount];
-	consecutivePastes = 1;
+        [self copyToPasteboard:[NSPasteboard generalPasteboard]];
+        lastPastedChangeCount = [[NSPasteboard generalPasteboard] changeCount];
+        lastCopiedChangeCount = [[NSPasteboard generalPasteboard] changeCount];
+        consecutivePastes = 1;
         for (l=[slayList count]-1; l>=0; l--)
         {   NSMutableArray	*list = [slayList objectAtIndex:l];
 
@@ -360,6 +389,7 @@ NSArray *e2TypesDrawExports(void)
  * the slist itself.  Also registers the PostScript and TIFF types since
  * the GraphicView knows how to convert its internal type to PostScript
  * or TIFF via the write{PS,TIFF} methods.
+ * modified: 2005-09-23
  */
 - copyToPasteboard:(NSPasteboard *)pboard types:(NSArray *)typesList
 {
@@ -367,20 +397,24 @@ NSArray *e2TypesDrawExports(void)
     {   NSMutableArray	*types = [[[NSMutableArray alloc] initWithCapacity:1] autorelease];
         NSData		*data;
 
-	[types addObject:e2PboardType];
-	if (!typesList || e2IncludesType(typesList, NSPostScriptPboardType))
-	    [types addObject:NSPostScriptPboardType];
-	if (!typesList || e2IncludesType(typesList, NSTIFFPboardType))
-	    [types addObject:NSTIFFPboardType];
-	[pboard declareTypes:types owner:[self class]];
+        [types addObject:e2PboardType];
+        if (!typesList || e2IncludesType(typesList, NSPostScriptPboardType))
+            [types addObject:NSPostScriptPboardType];
+#ifdef PB_CANSERVE_PDF
+        if (!typesList || e2IncludesType(typesList, NSPDFPboardType))
+            [types addObject:NSPDFPboardType];
+#endif
+        if (!typesList || e2IncludesType(typesList, NSTIFFPboardType))
+            [types addObject:NSTIFFPboardType];
+        [pboard declareTypes:types owner:[self class]];
         data = [self copySelection];
         if (data)
             [pboard setData:data forType:e2PboardType];
-	//[self writeLinkToPasteboard:pboard types:typesList];
-	return self;
+        //[self writeLinkToPasteboard:pboard types:typesList];
+        return self;
     }
     else
-	return nil;
+        return nil;
 }
 
 - copyToPasteboard:(NSPasteboard *)pboard
